class DPNegativeCycles
{
	private String[]  nodes;
	private float[][] cost;

	private float[]   distance;
	private int[]     next;
	private int[]     negative;

    private static float INF = Float.POSITIVE_INFINITY;


	public DPNegativeCycles (String[] nodes, float[][] cost)
	{
		this.nodes = nodes;
		this.cost  = cost;
	}

	// toString

	public String toString ()
	{
		int       last = nodes.length-1;
		int       count = 0;
		boolean[] visited = new boolean[nodes.length];
		String    output = "";


		if (negative!=null) {

		// Negative cycles

			output += "\nNEGATIVE CYCLES\n";

			for (int i=0; i<last; i++) {

				int[] cycle = getCycle(i);

				if (cycle!=null) {

					if (!visited[cycle[0]]) {

						output += "Negative cycle: ";

						for (int j=0; j<cycle.length; j++) {
							output += nodes[cycle[j]] + " -> ";
							visited[cycle[j]] = true;
						}

						output += nodes[cycle[0]] + " (value = "+getCycleValue(cycle)+")\n";			
					}

					count++;
				}
			}

			if (count==0) {
				output += "None\n";
			}

		} else {

		// Shortest paths

		for (int i=0; i<last; i++) {
			output += nodes[i] + " -> " + nodes[last] + " (cost "+distance[i]+"): ";

			int current = i;

			while (next[current]!=-1) {
				output += nodes[current] + " -> ";
				current = next[current];
			}

			output += nodes[current]+"\n";

		}

		}



		return output;
	}


    // Cycle

	public int[] getCycle (int node)
	{
		int   current = node;
		int   visited[] = new int[nodes.length];
		int   counter = 1;
		int[] cycle = null;

		visited[node] = counter++;

		while ( (next[current]!=-1) && (visited[next[current]] == 0)) {
			current = next[current];
			visited[current] = counter++;
		}

		if (next[current]!=-1) {
			cycle = new int[ counter - visited[next[current]] ];

			current = next[current];

			for (int i=0; i<cycle.length; i++) {
				cycle[i] = current;
				current = next[current];
			}
		}

		return cycle;
	}

    public String getCycleString (int[] cycle)
	{
		String output="";

		for (int j=0; j<cycle.length; j++) {
			output += nodes[cycle[j]] + " -> ";
		}

		output += nodes[cycle[0]];		

		return output;
	}

	public float getCycleValue (int[] cycle)
	{
		float value = 0;

   	    for (int i=0; i<cycle.length; i++)
			value += cost[cycle[i]][cycle[(i+1)%cycle.length]];

		return value;
	}


	// Distances from all the nodes to the last one...

	public void BellmanFord ()
	{
		distance = new float[nodes.length];
		next     = new int[nodes.length];

		distance[nodes.length-1] = 0;
		next[nodes.length-1] = -1;

		for (int i=0; i<nodes.length-1; i++) {
			distance[i] = INF;
			next[i] = -1;
		}

		for (int k=1; k<nodes.length; k++)
  		    for (int i=0; i<nodes.length; i++)
			    for (int j=0; j<nodes.length; j++) {

			        if (distance[i] > cost[i][j] + distance[j]) {
					   distance[i] = cost[i][j] + distance[j];
					   next[i] = j;
			        }
			    }

	}



	// Augment graph

	private void augment ()
	{
		String[]  oldNodes = nodes;
		float[][] oldCost  = cost;

		nodes = new String[oldNodes.length+1];

		for (int i=0; i<oldNodes.length; i++)
			nodes[i] = oldNodes[i];

		nodes[oldNodes.length] = "t";


		cost = new float[oldNodes.length+1][oldNodes.length+1];

		for (int i=0; i<oldNodes.length; i++)
			for (int j=0; j<oldNodes.length; j++)
				cost[i][j] = oldCost[i][j];

		for (int i=0; i<=oldNodes.length; i++)
			cost[i][oldNodes.length] = 0;

		for (int i=0; i<oldNodes.length; i++)
			cost[oldNodes.length][i] = INF;

	}



	// Distances from all nodes to the last one...

	public void BellmanFordNegative ()
	{
		augment();

		distance = new float[nodes.length];
		next     = new int[nodes.length];
		negative = new int[nodes.length];

		distance[nodes.length-1] = 0;
		next[nodes.length-1] = -1;
		negative[nodes.length-1] = -1;

		for (int i=0; i<nodes.length-1; i++) {
			distance[i] = INF;
			next[i] = -1;
			negative[i] = -1;
		}

		for (int k=1; k<nodes.length; k++)
  		    for (int i=0; i<nodes.length; i++)
			    for (int j=0; j<nodes.length; j++) {

			        if (distance[i] > cost[i][j] + distance[j]) {

					   if (k==nodes.length-1) {
						   negative[i] = next[i];
					   }

					   distance[i] = cost[i][j] + distance[j];
					   next[i] = j;
			        }
			    }

	}


	public static void main(String[] args) 
	{
		// Example Kleinberg 6.21

		String[]  kleinberg6x21nodes = new String[]    {   "s", "u", "w", "v" };
		float[][] kleinberg6x21weights = new float[][] { {   0,   2, INF,   1 }, 
			                                             { INF,   0,   3, INF },
			                                             { INF, INF,   0,  -6 }, 
			                                             { INF, INF, INF,   0 } };

		DPNegativeCycles kleinberg6x21 = new DPNegativeCycles(kleinberg6x21nodes, kleinberg6x21weights);

		kleinberg6x21.BellmanFord();

		System.out.println(kleinberg6x21);



		// Example - Kleinberg 6.23

		String[]  kleinberg6x23nodes = new String[]    {   "a", "b", "c", "d", "e", "t" };
		float[][] kleinberg6x23weights = new float[][] { {   0,  -4, INF, INF, INF,  -3 }, 
			                                             { INF,   0, INF,  -1,  -2, INF },
			                                             { INF,   8,   0, INF, INF,   3 }, 
			                                             {   6, INF, INF,   0, INF,   4 },
			                                             { INF, INF,  -3, INF,   0,   2 },
			                                             { INF, INF, INF, INF, INF,   0 } };

		DPNegativeCycles kleinberg6x23 = new DPNegativeCycles(kleinberg6x23nodes, kleinberg6x23weights);

		kleinberg6x23.BellmanFord();

		System.out.println(kleinberg6x23);



		// Example 

		String[]  cycle3nodes = new String[]    {   "a", "b", "c" };
		float[][] cycle3weights = new float[][] { {   0,  -4, INF }, 
			                                      { INF,   0,   2 },
			                                      {   1,   8,   0 } };

		DPNegativeCycles cycle3 = new DPNegativeCycles(cycle3nodes, cycle3weights);

		cycle3.BellmanFordNegative();

		System.out.println(cycle3);



		// Exercise 13

		String[]  exercise13nodes = new String[]    {   "a", "b", "c", "d", "v" };
		float[][] exercise13weights = new float[][] { {   0,   5, INF, INF, -15 }, 
			                                          { INF,   0, INF, INF, INF },
			                                          { INF,  18,   0, INF,   2 },
			                                          { INF, INF,   6,   0, INF }, 
			                                          { INF, -23, INF, -11,   0 } };

		DPNegativeCycles exercise13 = new DPNegativeCycles(exercise13nodes, exercise13weights);

		exercise13.BellmanFordNegative();

		System.out.println(exercise13);



		// Exercise 14

		String[]  exercise14nodes = new String[]    {    "G",    "$",    "",    "",    "" };
		float[][] exercise14weights = new float[][] { {    1,   7/1f,   3/4f,    INF,    INF }, 
			                                          { 1/7f,      1,    INF,  8/10f, 400/9f },
			                                          { 4/3f,    INF,      1, 11/10f,    INF },
			                                          {  INF,    INF,    INF,      1,     56 }, 
			                                          {  INF, 9/400f,    INF,  1/56f,      1 } };

		// -log(x)

		for (int i=0; i<exercise14nodes.length; i++)
			for (int j=0; j<exercise14nodes.length; j++) {

			    if (exercise14weights[i][j]!=INF)
					exercise14weights[i][j] = - (float) Math.log(exercise14weights[i][j]);

		    }

		DPNegativeCycles exercise14 = new DPNegativeCycles(exercise14nodes, exercise14weights);

		exercise14.BellmanFordNegative();

		System.out.println(exercise14);

        int[] euroCycle = exercise14.getCycle(3);

		System.out.println(exercise14.getCycleString(euroCycle) + " [profit = " + pct(1/Math.exp(exercise14.getCycleValue(euroCycle))-1)+"]" );
	}

	private static String pct (double v)
	{
		return ""+(Math.round(v*10000)/100.0)+"%";
	}
}
